// IMKit-Anthy: A Qtopia InputMethod interface for Anthy
// Copyright (C) 2002-2004  YamaKen <yamaken@bp.iij4u.or.jp>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// $Name: IMKIT_0_4_0 $
// $Id: ui_anthy.cpp,v 1.6 2004/01/18 09:28:50 yamaken Exp $

//TODO: Obsolete this. Replace with generic keyfilter
//configuration mechanism.

#include "ui_anthy.h"


static ButtonMapper<>::keymapdef_t default_cand_win_btnmap[] = {
  {0, ""}
};

static QCharKeyMapper<>::keymapdef_t default_cand_win_keymap[] = {
  //{'a', "select_candidate0"},
  {'\0', ""}
};

static OneShotKeyMapper<>::keymapdef_t default_ctrl_x_map[] = {
  {'j', "select_map_hiragana"},
  {'q', "select_map_katakana"},
  {'l', "select_map_alpha"},
  {'L', "select_map_walpha"},

  //quit prefixed context
  {IMKIT_ASCII_MOD_CTRL('g'), "do_nothing"},

  //{'c', "purge_engine"},

  {'\0', ""}
};

static ButtonMapper<>::keymapdef_t default_editing_context_btnmap[] = {
  {(IMKIT_BTNMAPPER_MOD_SHIFT | Qt::Key_Left), "shrink_segment"},
  {(IMKIT_BTNMAPPER_MOD_SHIFT | Qt::Key_Right), "enlarge_segment"},
  {Qt::Key_Up, "prev_candidate"},
  {Qt::Key_Down, "next_candidate"},
  {Qt::Key_Right, "move_forward"},
  {Qt::Key_Left, "move_back"},
  //{Qt::Key_Backspace, "erase_prev"},  //QWSC-hѴ
  {Qt::Key_Delete, "erase_next"},
  //{Qt::Key_Return, "commit"},  //QWSC-mѴ
  {(IMKIT_BTNMAPPER_MOD_SHIFT | Qt::Key_F30), "quit_one_level"}, //SELECT

#ifdef IMKIT_STEAL_OK_AND_CANCEL_KEY
  //"OK""Cancel"ϥץꥱνλ˳ƤƤ
  //ΤǡȤ٤Ǥʤ
  {Qt::Key_F33, "commit"},  //"OK"
  {Qt::Key_Escape, "quit_one_level"},  //"Cancel"
#endif
  {0, ""}
};

static QCharKeyMapper<>::keymapdef_t default_editing_context_ctrl_keymap[] = {
  {IMKIT_ASCII_MOD_CTRL('i'), "shrink_segment"},
  {IMKIT_ASCII_MOD_CTRL('o'), "enlarge_segment"},
  {IMKIT_ASCII_MOD_CTRL('p'), "prev_candidate"},
  {IMKIT_ASCII_MOD_CTRL('n'), "next_candidate"},
  {IMKIT_ASCII_MOD_CTRL('f'), "move_forward"},
  {IMKIT_ASCII_MOD_CTRL('b'), "move_back"},
  {IMKIT_ASCII_MOD_CTRL('a'), "beginning_of_line"},
  {IMKIT_ASCII_MOD_CTRL('e'), "end_of_line"},
  {IMKIT_ASCII_MOD_CTRL('k'), "cut"},
  {IMKIT_ASCII_MOD_CTRL('h'), "erase_prev"},
  {IMKIT_ASCII_MOD_CTRL('d'), "erase_next"},
  //{IMKIT_ASCII_MOD_CTRL('u'), "clear"},
  //{IMKIT_ASCII_MOD_CTRL('y'), "paste"},
  {IMKIT_ASCII_MOD_CTRL('m'), "commit"},
  {IMKIT_ASCII_MOD_CTRL('j'), "commit"},
  {IMKIT_ASCII_MOD_CTRL('g'), "quit_one_level"},
  {'\0', ""}
};

static ButtonMapper<>::keymapdef_t default_global_btnmap[] = {
  {0, ""}
};

static QCharKeyMapper<>::keymapdef_t default_global_ctrl_keymap[] = {
  {' ', "input_space"},
  {IMKIT_ASCII_MOD_CTRL('x'), "activate_ctrl_x_map"},
  {'\0', ""}
};


CascadeKeyFilter *
AnthyGUI::create_keyfilter(void) {
  //typedef TemplateCommandWithPreArg<OneShotKeyMapper<>, bool> PrefixMapper;
  typedef TemplateCommandWithPreArg<Activatable, bool> PrefixMapper;

  ctrl_x_mapper = new OneShotKeyMapper<>(default_ctrl_x_map, *command_map());
  (*_command_map)["activate_ctrl_x_map"]
    //= new PrefixMapper(ctrl_x_mapper, &OneShotKeyMapper<>::activate,
    = new PrefixMapper(ctrl_x_mapper, &Activatable::activate,
                       "activate_ctrl_x_map", true);

  cand_win_keymapper
    = new QCharKeyMapper<>(default_cand_win_keymap, *command_map());
  graph_keymapper
    = new GraphCharMapper<>((*command_map())["self_input_char"]);
  global_ctrl_keymapper
    = new QCharKeyMapper<>(default_global_ctrl_keymap, *command_map());
  editing_context_ctrl_keymapper
    = new QCharKeyMapper<>(default_editing_context_ctrl_keymap,
                           *command_map());

  cand_win_btnmapper
    = new ButtonMapper<>(default_cand_win_btnmap, *command_map());
  global_btnmapper
    = new ButtonMapper<>(default_global_btnmap, *command_map());
  editing_context_btnmapper
    = new ButtonMapper<>(default_editing_context_btnmap, *command_map());

  //Ϣ
  *cand_win_btnmapper
    << *cand_win_keymapper
    << *ctrl_x_mapper
    << *editing_context_btnmapper
    << *editing_context_ctrl_keymapper
    << *global_btnmapper
    << *global_ctrl_keymapper
    << *graph_keymapper
    ;

  return cand_win_btnmapper;
}

void
AnthyGUI::update_preedit_state(PreeditState new_state) {
  switch (new_state) {
  case IMKIT_PREEDIT_ST_CONV:
  case IMKIT_PREEDIT_ST_CSEG:
  case IMKIT_PREEDIT_ST_EDIT:
    editing_context_ctrl_keymapper->activate();
    editing_context_btnmapper->activate();
    break;

  case IMKIT_PREEDIT_ST_NONE:
  default:
    cand_win_keymapper->deactivate();
    cand_win_btnmapper->deactivate();
    editing_context_ctrl_keymapper->deactivate();
    editing_context_btnmapper->deactivate();
  }
}

AnthyGUI::AnthyGUI(AnthyEngine *engine_init,
                   StdIMWidgetFactory *imwidget_factory_init)
  : StdIMUserInterface(engine_init, imwidget_factory_init),
    cand_win_keymapper(NULL), graph_keymapper(NULL),
    global_ctrl_keymapper(NULL), editing_context_ctrl_keymapper(NULL),
    ctrl_x_mapper(NULL),
    cand_win_btnmapper(NULL),
    global_btnmapper(NULL), editing_context_btnmapper(NULL)
{
}

void
AnthyGUI::set_activity(bool activity) {
  //TODO: activate()engine̵äassemble
  ctrl_x_mapper->deactivate();
  graph_keymapper->activate(activity);
  cand_win_keymapper->deactivate();
  cand_win_btnmapper->deactivate();
  global_ctrl_keymapper->activate(activity);
  global_btnmapper->activate(activity);
  editing_context_ctrl_keymapper->deactivate();
  editing_context_btnmapper->deactivate();
  if (!activity && imwidget()) {
    cand_win->deactivate();
  }
}

bool
AnthyGUI::is_active(void) const {
  return (cand_win_keymapper->is_active() || graph_keymapper->is_active());
}
